@@ -1,18 +1,17 @@
+# frozen_string_literal: true
+
 require 'spec_helper_acceptance'
 
 describe 'concat ensure_newline parameter' do
-  basedir = default.tmpdir('concat')
-  context 'when false' do
-    before(:all) do
-      pp = <<-MANIFEST
-        file { '#{basedir}':
-          ensure => directory
-        }
-      MANIFEST
+  attr_reader :basedir
 
-      apply_manifest(pp)
-    end
-    pp = <<-MANIFEST
+  before(:all) do
+    @basedir = setup_test_directory
+  end
+
+  describe 'when false' do
+    let(:pp) do
+      <<-MANIFEST
       concat { '#{basedir}/file':
         ensure_newline => false,
       }
@@ -25,20 +24,18 @@ describe 'concat ensure_newline parameter' do
         content => '2',
       }
     MANIFEST
-
-    it 'applies the manifest twice with no stderr' do
-      apply_manifest(pp, catch_failures: true)
-      apply_manifest(pp, catch_changes: true)
     end
 
-    describe file("#{basedir}/file") do
-      it { is_expected.to be_file }
-      its(:content) { is_expected.to match '12' }
+    it 'applies the manifest twice with no stderr' do
+      idempotent_apply(pp)
+      expect(file("#{basedir}/file")).to be_file
+      expect(file("#{basedir}/file").content).to match '12'
     end
   end
 
-  context 'when true' do
-    pp = <<-MANIFEST
+  describe 'when true' do
+    let(:pp) do
+      <<-MANIFEST
       concat { '#{basedir}/file':
         ensure_newline => true,
       }
@@ -51,17 +48,12 @@ describe 'concat ensure_newline parameter' do
         content => '2',
       }
     MANIFEST
-
-    it 'applies the manifest twice with no stderr' do
-      apply_manifest(pp, catch_failures: true)
-      apply_manifest(pp, catch_changes: true)
     end
 
-    describe file("#{basedir}/file") do
-      it { is_expected.to be_file }
-      its(:content) do
-        is_expected.to match %r{1\n2\n}
-      end
+    it 'applies the manifest twice with no stderr' do
+      idempotent_apply(pp)
+      expect(file("#{basedir}/file")).to be_file
+      expect(file("#{basedir}/file").content).to match %r{1\r?\n2\r?\n}
     end
   end
 end
